@@ -133,11 +133,6 @@ int gfxloader_gfx1(tVM68k_ubyte* gfxbuf,tVM68k_ulong gfxsize,tVM68k_ubyte versio
 
 }
 
-// the gfx2 format stored some values as little endian.
-#define READ_INT16LE(ptr,idx)   (\
-        (((tVM68k_ulong)((ptr)[((idx)+1)])&0xff)<< 8)   |\
-        (((tVM68k_ulong)((ptr)[((idx)+0)])&0xff)<< 0)   |\
-        0)
 // the gfx2 format introduced MIXED ENDIAN. 
 #define READ_INT32ME(ptr,idx)   (\
         (((tVM68k_ulong)((ptr)[((idx)+1)])&0xff)<<24)   |\
@@ -263,13 +258,6 @@ int gfxloader_gfx2(tVM68k_ubyte* gfxbuf,tVM68k_ulong gfxsize,tVM68k_ubyte versio
 }
 
 
-// the gfx3 format stored some values as little endian.
-#define READ_INT32LE(ptr,idx)   (\
-        (((tVM68k_ulong)((ptr)[((idx)+3)])&0xff)<<24)   |\
-        (((tVM68k_ulong)((ptr)[((idx)+2)])&0xff)<<16)   |\
-        (((tVM68k_ulong)((ptr)[((idx)+1)])&0xff)<< 8)   |\
-        (((tVM68k_ulong)((ptr)[((idx)+0)])&0xff)<< 0)   |\
-        0)
 #define	MAXPICWIDTH	512
 int gfxloader_gfx3(tVM68k_ubyte* gfxbuf,tVM68k_ulong gfxsize,tVM68k_ubyte version,int picnum,tPicture* pPicture)
 {
@@ -327,6 +315,7 @@ int gfxloader_gfx3(tVM68k_ubyte* gfxbuf,tVM68k_ulong gfxsize,tVM68k_ubyte versio
 	// in case the offset is -1, it must be in the other one.
 	offs1=(tVM68k_slong)READ_INT32LE(gfxbuf,indexoffs+picnum*4);
 	offs2=(tVM68k_slong)READ_INT32LE(gfxbuf,indexoffs+indexlen/2+picnum*4);
+	if (picnum!=30 && offs1!=-1 && offs2!=-1) offs1=-1;	// in case one picture is stored on both disks, prefer the second one. (due to reasons.)
 
 	if (picnum==30 && offs1==-1 && offs2==-1) offs1=0;	// special case: the title screen for the GUILD of thieves is the first picture in DISK1.PIX
 	if (offs1!=-1) offset=offs1+disk1offs;			// in case the index was found in the first half, use disk1
@@ -507,9 +496,259 @@ int gfxloader_gfx3(tVM68k_ubyte* gfxbuf,tVM68k_ulong gfxsize,tVM68k_ubyte versio
 
 	return retval;
 }
+// the gfx4 format is used to handle the pictures from the Magnetic Windows system.
+// just like the gfx2, it starts with a directory. 6 byte picname, 4 bytes (little endian) offset, 4 bytes (little endian) length.
+// at the offset, the picture is being comprised of the tree (type 7) and image (type 6) from the Magnetic Windows resource files.
+// the tree is ALWAYS 609 bytes long. The size of the image varies.
+//
+// the huffman tree uses 9 bits to encode 8 bits: the first 32 bytes are a bitmask (MSB first). Then the branches/symbols follow. 
+// if the bit from the bitmask is set, it is a terminal symbol.
+// 0x00...0x1f:  LEFTMASK		(0)
+// 0x20...0x11f: LEFTBRANCH
+// 0x120..0x13f: RIGHTMASK		(1)
+// 0x140..0x23f: RIGHTBRANCH
+// Byte 0x240    escape character (for run level encoding)
+// Byte 0x241...0x250: EGA palette
+// Byte 0x251...0x260: EGA palette pairs
+//
+//
+// the image data looks like this:
+// 0x00..0x03: magic header
+// 0x04..0x23: 16x2 bytes RGB values (4 bits per channel, little endian, 0x0rgb)
+// 0x24..0x25: width
+// 0x26..0x27: height
+// 0x28..0x29: transparency placeholder
+// 0x2a..0x2b: size
+// 0x2c......: bit stream. MSB first. when the bit is set (=1), follow the RIGHT branch.
+//
+// the run level encoding is signalled by the escape character from the tree (byte 0x240).
+// in case the next character is 0xff, it actually an honest escape character.
+// if any other value follows, it is the repeat num.
+// the character AFTER that one is being repeated (4+repeat) number of times.
+//
+// each symbol is 8 bits large, 4 bits are the pixel. CAREFUL: bits 0..3 are the left, bits 4..7 are the right pixel
+// in case the width is not divisible by 2, bits 4..7 are being ignored.
+//
+// the xor is being performed line by line.
+//
+// note that the end of the image comes BEFORE the end of the bitstream. (due to a bug in the encoder)
+int gfxloader_gfx4(tVM68k_ubyte* gfxbuf,tVM68k_ulong gfxsize,tVM68k_ubyte version,tVM68k_ubyte* picname,tPicture* pPicture,int egamode)
+{
+#define	SIZEOFTREE	609
+	int directorysize;
+	int retval;
+	int found;
+	int i;
+	int offset,length;
+	int offset_vanilla,length_vanilla;
+	int offset_ega,length_ega;
+	int offset_anim,length_anim;
+	int j;
+	pPicture->width=0;
+	pPicture->height=0;
+	pPicture->halftones=0;	// only gfx3 offers halftone pictures
+	// the gfx4 buffer starts with the magic value, and then a directory
+	retval=0;
+	found=0;
+	directorysize=READ_INT16BE(gfxbuf,4);	
+	offset=offset_ega=offset_anim=offset_vanilla=-1;
+	length=length_ega=length_anim=length_vanilla=-1;
+	for (i=0;i<directorysize && !found;i+=14)
+	{
+		tVM68k_ubyte c1,c2;
+		found=1;
+		j=0;
+		do
+		{
+			c1=gfxbuf[6+i+j];
+			c2=picname[j];
+			if ((c1&0x5f)!=(c2&0x5f)) found=0;
+			if ((c1&0x5f)==0) j=6;	// end of entry reached.
+			j++;
+		} while (j<6 && found);
+		if (found)
+		{
+			int ega;
+			int stillimage;
+#define	STILLMAGIC	0x00005ed0
+			const unsigned short egapalette[16]={0x000,0x005,0x050,0x055, 0x500,0x505,0x550,0x555, 0x222,0x007,0x070,0x077, 0x700,0x707,0x770,0x777};
+			offset=READ_INT32LE(gfxbuf,i+6+6);
+			length=READ_INT32LE(gfxbuf,i+6+10);
+
+			// check if the image is not the ega image, and it is not an animation background.	
+
+			// the EGA images have a specific RGB palette
+			// if not, it is not a EGA image
+			ega=16;
+			for (j=0;j<16;j++)
+			{
+				if (READ_INT16LE(gfxbuf,offset+SIZEOFTREE+4+2*j)!=egapalette[j]) ega--;	
+			}
+			ega=(ega>=15);
+
+			stillimage=1;
+			if (READ_INT32LE(gfxbuf,offset+length-4)!=STILLMAGIC) stillimage=0;		// if they do not, it is an animation
+			found=0;
+			if (!ega && stillimage)
+			{
+				offset_vanilla=offset;
+				length_vanilla=length;
+			}
+
+			if (ega) 
+			{
+				offset_ega=offset;
+				length_ega=length;
+			}
+			if (!stillimage)
+			{
+				offset_anim=offset;
+				length_anim=length;
+			}
+		}
+	}
+	// in case the image was not found
+	offset=-1;
+	length=-1;
+	if (egamode)
+	{
+		offset=offset_ega;
+		length=length_ega;
+	}
+	if (offset==-1)
+	{
+		offset=offset_vanilla;
+		length=length_vanilla;
+	}
+	if (offset==-1)
+	{
+		offset=offset_anim;
+		length=length_anim;
+	}
+	if (offset==-1)
+	{
+		offset=offset_ega;
+		length=length_ega;
+	}
+	if (offset!=-1 && length!=-1) found=1;
+	
+	if (found)
+	{
+		int treestart;
+		int picstart;
+		int size;
+		int treeidx;
+		int repnum;
+		int rlestate;	// for the run length encoding. state 0: if not the escape char, output. otherwise -> state 1. in state 1, if the symbol is 0xff, the output is the escapechar. otherwise, the repition number (sans 4) -> state 4. in state 4, repeat the symbol	
+		tVM68k_ubyte escapechar;
+		tVM68k_ubyte byte;
+		tVM68k_ubyte mask;
+		treestart=offset+0;
+		picstart=offset+SIZEOFTREE;
 
 
-int gfxloader_unpackpic(tVM68k_ubyte* gfxbuf,tVM68k_ulong gfxsize,tVM68k_ubyte version,int picnum,tVM68k_ubyte* picname,tPicture* pPicture)
+		// byte 0x240 in the tree is the escape symbol for the run level encoding
+		escapechar=gfxbuf[treestart+0x240];
+		// bytes 0x04..0x23: RGB values
+		for (i=0;i<16;i++)
+		{
+			pPicture->palette[i]=READ_INT16LE(gfxbuf,picstart+0x4+i*2);
+		}
+		// bytes 0x24,0x25= width
+		// bytes 0x26,0x27= height
+		pPicture->width=READ_INT16LE(gfxbuf,picstart+0x24);
+		pPicture->height=READ_INT16LE(gfxbuf,picstart+0x26);
+		// bytes 0x2a,0x2b= size of the bitstream (in bytes)
+		size=READ_INT16LE(gfxbuf,picstart+0x2a);
+		j=0;
+		treeidx=0;
+		mask=0;
+		i=0;	
+		rlestate=0;
+		repnum=1;
+
+		// i is counting up the bytes in the bitstream.
+		// j is counting up the pixels of the image		
+		while (((i<(length-SIZEOFTREE) && i<size ) || mask ) && j<(pPicture->width*pPicture->height))
+		{
+			tVM68k_ubyte lterm,rterm,term;
+			tVM68k_ubyte lbranch,rbranch,branch;
+			// the bitmask is denoting (MSB first) if an entry is a terminal symbol (=1) or a branch (=0)
+			lterm=gfxbuf[treestart+0x00 +treeidx/8]&(0x80>>(treeidx%8));
+			rterm=gfxbuf[treestart+0x120+treeidx/8]&(0x80>>(treeidx%8));
+
+			// the entry in the table could either be a branch or a terminal symbol
+			lbranch=gfxbuf[treestart+0x20 +treeidx];
+			rbranch=gfxbuf[treestart+0x140+treeidx];
+
+			if (mask==0)
+			{
+				mask=0x80;
+				byte=gfxbuf[picstart+i+0x2c];
+				i++;
+			}
+		
+			term  =(byte&mask)?  rterm:lterm;
+			branch=(byte&mask)?rbranch:lbranch;
+			mask>>=1;
+		
+
+			if (term)
+			{
+				if (rlestate==0)
+				{
+					if (branch==escapechar) 
+					{
+						rlestate=1;
+					} else {
+						repnum=1;	
+					}
+				} else if (rlestate==1) {
+					if (branch==0xff)
+					{
+						branch=escapechar;
+						repnum=1;
+						rlestate=0;
+					} else {
+						repnum=branch+4;	// this form of RLE makes sense when the same byte was repeated 4 or more times in the source picture
+						rlestate=2;
+					}
+				} else if (rlestate==2) {
+					rlestate=0;	// the current entry is a terminal symbol, which is going to be repeated
+				}
+				if (rlestate==0)
+				{
+					while (repnum && j<(pPicture->width*pPicture->height))
+					{
+						// the lower 4 bits are the LEFT pixel
+						pPicture->pixels[j]=branch&0xf;
+						j++;
+						// one byte holds two pixels. but when the width is not divisible by 2, drop the remaining nibble.
+						if (j%pPicture->width) // the higher 4 bits are the RIGHT pixel; but only if it is not outside the scope of the image.
+						{
+							pPicture->pixels[j]=(branch>>4)&0xf;	// the higher 4 bytes are the RIGHT pixel
+							j++;
+						}
+						repnum--;
+					}
+				}
+				treeidx=0;	// go back to the start;
+			} else {	// not a terminal symbol. keep following the branches
+				treeidx=branch;
+			}
+		}
+		// the finishing touch: XOR each line with the previous one
+		for (i=pPicture->width;i<pPicture->height*pPicture->width;i++)
+		{
+			pPicture->pixels[i]^=pPicture->pixels[i-pPicture->width];
+		}
+	}
+	
+
+	return retval;
+}
+
+int gfxloader_unpackpic(tVM68k_ubyte* gfxbuf,tVM68k_ulong gfxsize,tVM68k_ubyte version,int picnum,tVM68k_ubyte* picname,tPicture* pPicture,int egamode)
 {
 	int retval;
 
@@ -519,6 +758,7 @@ int gfxloader_unpackpic(tVM68k_ubyte* gfxbuf,tVM68k_ulong gfxsize,tVM68k_ubyte v
 	if (gfxbuf[0]=='M' && gfxbuf[1]=='a' && gfxbuf[2]=='P' && gfxbuf[3]=='i') retval=gfxloader_gfx1(gfxbuf,gfxsize,version,picnum,pPicture);
 	if (gfxbuf[0]=='M' && gfxbuf[1]=='a' && gfxbuf[2]=='P' && gfxbuf[3]=='2') retval=gfxloader_gfx2(gfxbuf,gfxsize,version,picname,pPicture);
 	if (gfxbuf[0]=='M' && gfxbuf[1]=='a' && gfxbuf[2]=='P' && gfxbuf[3]=='3') retval=gfxloader_gfx3(gfxbuf,gfxsize,version,picnum,pPicture);
+	if (gfxbuf[0]=='M' && gfxbuf[1]=='a' && gfxbuf[2]=='P' && gfxbuf[3]=='4') retval=gfxloader_gfx4(gfxbuf,gfxsize,version,picname,pPicture,egamode);
 	return retval;
 }
 int gfxloader_picture_calcxpmsize(tPicture* pPicture,int* xpmsize)
